#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>

#if defined (BSD) && !_POSIX_SOURCE
#   include <sys/dir.h>
    typedef struct direct Dirent;
#else
#     include <dirent.h>
    typedef struct dirent Dirent;
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "utils.h"

/* The support of ESD do not work yet. */

#ifdef ESD
#  include <esd.h>
#endif /* ESD */

#define MAX_PATH 256

gint iceEventNum = -1;
gint soundFileNum = -1;

char program_name[1024];

void message_box(gchar *message)
{
    GtkWidget *message_dialog = NULL;
    GtkLabel  *warning_label;
    if(!message_dialog)
        message_dialog = create_dialog1();
    warning_label = GTK_LABEL(lookup_widget(GTK_WIDGET(message_dialog), "warning_label"));
    gtk_label_set_text(warning_label, message);
    gtk_widget_show(message_dialog);
} /* message_box */

static void playSoundFile(const char *soundFile)
{
    char buf[MAX_PATH + 30];
    int ofd;
#ifdef ESD
    if(access(soundFile, R_OK))
    {
        sprintf(buf, "Cann't open '%s' !", soundFile);
        message_box(buf);
        return;
    }

    ofd = open("/dev/dsp", O_WRONLY);
    if (ofd == -1)
    {
          message_box("Cann't write in '/dev/dsp' !");
          return;
    }

    close(ofd);
    esd_play_file(program_name, soundFile, 1);
#else
    int ifd;
    int n;

    ifd = open(soundFile, O_RDONLY);
    if (ifd == -1)
    {

          sprintf(buf, "Cann't open '%s' !", soundFile);
          message_box(buf);
          return;
    }

    ofd = open("/dev/dsp", O_WRONLY);
    if (ofd == -1)
    {
          message_box("Cann't open '/dev/dsp' !");
          return;
    }
			
    while ((n = read(ifd, buf, sizeof(buf))) > 0)
          write(ofd, buf, n);

    close(ifd);
    close(ofd);
#endif
} /* playSoundFile */

void
on_eventsCList_select_row              (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    iceEventNum = row;
}


void
on_butPlayEventSound_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
    gchar *eventName;
    char  eventSoundFile[MAX_PATH];

    if(iceEventNum == -1)
        return;

    gtk_clist_get_text(GTK_CLIST(lookup_widget(GTK_WIDGET(button), "eventsCList")),
                       iceEventNum, 0, &eventName);
    sprintf(eventSoundFile, "%s/%s.wav",
            gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button), "edIceSoundsDir"))),
            eventName);

    playSoundFile(eventSoundFile);

}

void
on_dirsCList_select_row                (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    soundFileNum = row;
}

void
on_butTest_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
    gchar *soundName;
    char  soundFile[MAX_PATH];

    if(soundFileNum == -1)
        return;

    gtk_clist_get_text(GTK_CLIST(lookup_widget(GTK_WIDGET(button), "dirsCList")),
                       soundFileNum, 0, &soundName);
    sprintf(soundFile, "%s/%s",
            gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button), "edSoundsDir"))),
            soundName);

    playSoundFile(soundFile);
}

void
on_butReload_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkCList *list;
    Dirent   *dp;
    DIR      *dir_fdesc;
    gchar    *fileNamePtr[1];
    int      nameLen;
    gchar    *dirname;
    char     buf[MAX_PATH + 50];

    list = GTK_CLIST(lookup_widget(GTK_WIDGET(button), "dirsCList"));

    dir_fdesc = opendir(dirname = gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button),
                                                                  "edSoundsDir"))));
    if(!dir_fdesc)
    {
        sprintf(buf, " Cann't read '%s' ! ", dirname);
        message_box(buf);
        return;
    }

    gtk_clist_clear(list);
    gtk_clist_freeze(list);

    /* scaning of directory */
    while(dp = readdir(dir_fdesc))
    {
        if(dp->d_name[0] == '.')
            continue;

        nameLen = strlen(dp->d_name);
        if(nameLen < 5)
            continue;
        if(strcmp(dp->d_name + nameLen - 4, ".wav"))
            continue;

        fileNamePtr[0] = dp->d_name;
        gtk_clist_append(list, fileNamePtr);
    }/* while(dp = readdir(dir_fdesc)) */
    closedir(dir_fdesc);
    gtk_clist_thaw(list);
    soundFileNum = -1;
}/* on_butReload_clicked */


void
on_butUnSet_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
    gchar *eventName;
    char  eventSoundFile[MAX_PATH];
    char  buf[MAX_PATH];

    if(iceEventNum == -1)
        return;

    gtk_clist_get_text(GTK_CLIST(lookup_widget(GTK_WIDGET(button), "eventsCList")),
                       iceEventNum, 0, &eventName);
    sprintf(eventSoundFile, "%s/%s.wav",
            gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button), "edIceSoundsDir"))),
            eventName);
    if(unlink(eventSoundFile) == -1)
    {
        sprintf(buf, "Cann't remove '%s' !", eventSoundFile);
        message_box(buf);
    }
}


void
on_butSet_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
    char  eventSoundFile[MAX_PATH];
    gchar *soundName;
    char  soundFile[MAX_PATH];
    int ifd;
    int ofd;
    int n;
    char buf[1024];


    if(soundFileNum == -1 || iceEventNum == -1)
        return;

    gtk_clist_get_text(GTK_CLIST(lookup_widget(GTK_WIDGET(button), "eventsCList")),
                       iceEventNum, 0, &soundName);
    sprintf(eventSoundFile, "%s/%s.wav",
            gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button), "edIceSoundsDir"))),
            soundName);

    gtk_clist_get_text(GTK_CLIST(lookup_widget(GTK_WIDGET(button), "dirsCList")),
                       soundFileNum, 0, &soundName);
    sprintf(soundFile, "%s/%s",
            gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(button), "edSoundsDir"))),
            soundName);

    ifd = open(soundFile, O_RDONLY);
    if (ifd == -1)
    {
          sprintf(buf, "Cann't open '%s' !", soundFile);
          message_box(buf);
          return;
    }
    ofd = open(eventSoundFile, O_WRONLY | O_CREAT);
    if (ofd == -1)
    {
          sprintf(buf, "Cann't open '%s' !", eventSoundFile);
          message_box(buf);
          close(ifd);
          return;
    }
			
    while ((n = read(ifd, buf, sizeof(buf))) > 0)
          write(ofd, buf, n);

    fchmod(ofd, S_IREAD | S_IWRITE | S_IRGRP | S_IROTH);

    close(ifd);
    close(ofd);

}/* on_butSet_clicked */


void
destroy                                (GtkObject       *object,
                                        gpointer         user_data)
{
    gtk_main_quit();
}


gboolean
delete_event                           (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{

  return FALSE;
}


void
on_butHelp_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *help_dialog = NULL;
    if(!help_dialog)
        help_dialog = create_dialog_help();
    gtk_widget_show(help_dialog);
}


GtkWidget *dirSelector = 0;
GtkEntry  *dirEntry    = 0;
gint      isSoundDir   = 0;

void dir_ok_sel (GtkWidget *w, GtkFileSelection *fs)
{
    gtk_entry_set_text(dirEntry, gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
    on_butReload_clicked(GTK_WIDGET(dirEntry), NULL);
    isSoundDir = 0;
} /* dir_ok_sel */

void create_dirSelector()
{
    dirSelector = gtk_file_selection_new (_("Directory selection"));
    gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
                                                 (dirSelector)->cancel_button),
                                    "clicked", (GtkSignalFunc) gtk_widget_hide,
                                    GTK_OBJECT (dirSelector));

    gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (dirSelector)->ok_button),
                             "clicked", (GtkSignalFunc) dir_ok_sel, dirSelector );

    gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
                                              (dirSelector)->ok_button),
                              "clicked", (GtkSignalFunc) gtk_widget_hide,
                               GTK_OBJECT (dirSelector));
} /* create_dirSelector */


void
on_butIceSndDir_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    if(!dirSelector)
        create_dirSelector();

    dirEntry = GTK_ENTRY(lookup_widget(GTK_WIDGET(button), "edIceSoundsDir"));
    gtk_widget_show(dirSelector);
} /* on_butIceSndDir_clicked */

void
on_butSndDir_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    if(!dirSelector)
        create_dirSelector();

    dirEntry = GTK_ENTRY(lookup_widget(GTK_WIDGET(button), "edSoundsDir"));
    isSoundDir = 1;
    gtk_widget_show(dirSelector);
} /* on_butSndDir_clicked */